@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: graphics.cpp 48816 2007-09-19 23:21:51Z RD $
+// RCS-ID:      $Id: graphics.cpp 56826 2008-11-17 21:39:29Z KO $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -23,6 +23,7 @@
     #include "wx/image.h"
 #endif
 
+#include "wx/strconv.h"
 #include "wx/mac/uma.h"
 
 #ifdef __MSL__
@@ -479,60 +480,6 @@ void wxMacCoreGraphicsPenData::Apply( wxGraphicsContext* context )
 // Brush
 //
 
-class wxMacCoreGraphicsBrushData : public wxGraphicsObjectRefData
-{
-public:
-    wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer );
-    wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer, const wxBrush &brush );
-    ~wxMacCoreGraphicsBrushData ();
-
-    virtual void Apply( wxGraphicsContext* context );
-    void CreateLinearGradientBrush( wxDouble x1, wxDouble y1, wxDouble x2, wxDouble y2,
-        const wxColour&c1, const wxColour&c2 );
-    void CreateRadialGradientBrush( wxDouble xo, wxDouble yo, wxDouble xc, wxDouble yc, wxDouble radius,
-    const wxColour &oColor, const wxColour &cColor );
-
-    virtual bool IsShading() { return m_isShading; }
-    CGShadingRef GetShading() { return m_shading; }
-protected:
-    CGFunctionRef CreateGradientFunction( const wxColour& c1, const wxColour& c2 );
-    static void CalculateShadingValues (void *info, const CGFloat *in, CGFloat *out);
-    virtual void Init();
-
-    wxMacCFRefHolder<CGColorRef> m_color;
-    wxMacCFRefHolder<CGColorSpaceRef> m_colorSpace;
-
-    bool m_isPattern;
-    wxMacCFRefHolder<CGPatternRef> m_pattern;
-    CGFloat* m_patternColorComponents;
-
-    bool m_isShading;
-    CGFunctionRef m_gradientFunction;
-    CGShadingRef m_shading;
-    CGFloat *m_gradientComponents;
-};
-
-wxMacCoreGraphicsBrushData::wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer) : wxGraphicsObjectRefData( renderer )
-{
-    Init();
-}
-
-void wxMacCoreGraphicsBrushData::CreateLinearGradientBrush( wxDouble x1, wxDouble y1, wxDouble x2, wxDouble y2,
-        const wxColour&c1, const wxColour&c2 )
-{
-    m_gradientFunction = CreateGradientFunction( c1, c2 );
-    m_shading = CGShadingCreateAxial( wxMacGetGenericRGBColorSpace(), CGPointMake(x1,y1), CGPointMake(x2,y2), m_gradientFunction, true, true ) ;
-    m_isShading = true ;
-}
-
-void wxMacCoreGraphicsBrushData::CreateRadialGradientBrush( wxDouble xo, wxDouble yo, wxDouble xc, wxDouble yc, wxDouble radius,
-    const wxColour &oColor, const wxColour &cColor )
-{
-    m_gradientFunction = CreateGradientFunction( oColor, cColor );
-    m_shading = CGShadingCreateRadial( wxMacGetGenericRGBColorSpace(), CGPointMake(xo,yo), 0, CGPointMake(xc,yc), radius, m_gradientFunction, true, true ) ;
-    m_isShading = true ;
-}
-
 static const char *gs_stripedback_xpm[] = {
 /* columns rows colors chars-per-pixel */
 "4 4 2 1",
@@ -547,10 +494,40 @@ static const char *gs_stripedback_xpm[] = {
 
 wxBitmap gs_stripedback_bmp( wxImage( (const char* const* ) gs_stripedback_xpm  ), -1 ) ;
 
-wxMacCoreGraphicsBrushData::wxMacCoreGraphicsBrushData(wxGraphicsRenderer* renderer, const wxBrush &brush) : wxGraphicsObjectRefData( renderer )
+wxMacCoreGraphicsColour::~wxMacCoreGraphicsColour()
+{
+    delete[] m_patternColorComponents;
+}
+
+void wxMacCoreGraphicsColour::Init()
+{
+    m_isPattern = false;
+    m_patternColorComponents = NULL;
+}
+
+void wxMacCoreGraphicsColour::Apply( CGContextRef cgContext )
+{
+    if ( m_isPattern )
+    {
+        CGAffineTransform matrix = CGContextGetCTM( cgContext );
+        CGContextSetPatternPhase( cgContext, CGSizeMake(matrix.tx, matrix.ty) );
+        CGContextSetFillColorSpace( cgContext , m_colorSpace );
+        CGContextSetFillPattern( cgContext, m_pattern , m_patternColorComponents );
+    }
+    else
+    {
+        CGContextSetFillColorWithColor( cgContext, m_color );
+    }
+}
+
+wxMacCoreGraphicsColour::wxMacCoreGraphicsColour() 
 {
     Init();
+}
 
+wxMacCoreGraphicsColour::wxMacCoreGraphicsColour( const wxBrush &brush )
+{
+    Init();
     if ( brush.GetStyle() == wxSOLID )
     {
         if ( brush.MacGetBrushKind() == kwxMacBrushTheme )
@@ -620,6 +597,62 @@ wxMacCoreGraphicsBrushData::wxMacCoreGraphicsBrushData(wxGraphicsRenderer* rende
     }
 }
 
+class wxMacCoreGraphicsBrushData : public wxGraphicsObjectRefData
+{
+public:
+    wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer );
+    wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer, const wxBrush &brush );
+    ~wxMacCoreGraphicsBrushData ();
+
+    virtual void Apply( wxGraphicsContext* context );
+    void CreateLinearGradientBrush( wxDouble x1, wxDouble y1, wxDouble x2, wxDouble y2,
+        const wxColour&c1, const wxColour&c2 );
+    void CreateRadialGradientBrush( wxDouble xo, wxDouble yo, wxDouble xc, wxDouble yc, wxDouble radius,
+    const wxColour &oColor, const wxColour &cColor );
+
+    virtual bool IsShading() { return m_isShading; }
+    CGShadingRef GetShading() { return m_shading; }
+protected:
+    CGFunctionRef CreateGradientFunction( const wxColour& c1, const wxColour& c2 );
+    static void CalculateShadingValues (void *info, const CGFloat *in, CGFloat *out);
+    virtual void Init();
+
+    wxMacCoreGraphicsColour m_cgColor;
+
+    bool m_isShading;
+    CGFunctionRef m_gradientFunction;
+    CGShadingRef m_shading;
+    CGFloat *m_gradientComponents;
+};
+
+wxMacCoreGraphicsBrushData::wxMacCoreGraphicsBrushData( wxGraphicsRenderer* renderer) : wxGraphicsObjectRefData( renderer )
+{
+    Init();
+}
+
+void wxMacCoreGraphicsBrushData::CreateLinearGradientBrush( wxDouble x1, wxDouble y1, wxDouble x2, wxDouble y2,
+        const wxColour&c1, const wxColour&c2 )
+{
+    m_gradientFunction = CreateGradientFunction( c1, c2 );
+    m_shading = CGShadingCreateAxial( wxMacGetGenericRGBColorSpace(), CGPointMake(x1,y1), CGPointMake(x2,y2), m_gradientFunction, true, true ) ;
+    m_isShading = true ;
+}
+
+void wxMacCoreGraphicsBrushData::CreateRadialGradientBrush( wxDouble xo, wxDouble yo, wxDouble xc, wxDouble yc, wxDouble radius,
+    const wxColour &oColor, const wxColour &cColor )
+{
+    m_gradientFunction = CreateGradientFunction( oColor, cColor );
+    m_shading = CGShadingCreateRadial( wxMacGetGenericRGBColorSpace(), CGPointMake(xo,yo), 0, CGPointMake(xc,yc), radius, m_gradientFunction, true, true ) ;
+    m_isShading = true ;
+}
+
+wxMacCoreGraphicsBrushData::wxMacCoreGraphicsBrushData(wxGraphicsRenderer* renderer, const wxBrush &brush) : wxGraphicsObjectRefData( renderer ),
+    m_cgColor( brush )
+{
+    Init();
+
+}
+
 wxMacCoreGraphicsBrushData::~wxMacCoreGraphicsBrushData()
 {
     if ( m_shading )
@@ -629,15 +662,12 @@ wxMacCoreGraphicsBrushData::~wxMacCoreGraphicsBrushData()
         CGFunctionRelease(m_gradientFunction);
 
     delete[] m_gradientComponents;
-    delete[] m_patternColorComponents;
 }
 
 void wxMacCoreGraphicsBrushData::Init()
 {
-    m_patternColorComponents = NULL;
     m_gradientFunction = NULL;
     m_shading = NULL;
-    m_isPattern = false;
     m_gradientComponents = NULL;
     m_isShading = false;
 }
@@ -652,17 +682,7 @@ void wxMacCoreGraphicsBrushData::Apply( wxGraphicsContext* context )
     }
     else
     {
-        if ( m_isPattern )
-        {
-            CGAffineTransform matrix = CGContextGetCTM( cg );
-            CGContextSetPatternPhase( cg, CGSizeMake(matrix.tx, matrix.ty) );
-            CGContextSetFillColorSpace( cg , m_colorSpace );
-            CGContextSetFillPattern( cg, m_pattern , m_patternColorComponents );
-        }
-        else
-        {
-            CGContextSetFillColorWithColor( cg, m_color );
-        }
+        m_cgColor.Apply( cg );
     }
 }
 
@@ -709,11 +729,20 @@ public:
     ~wxMacCoreGraphicsFontData();
 
     virtual ATSUStyle GetATSUStyle() { return m_macATSUIStyle; }
+#if !wxUSE_UNICODE
+    const wxMBConv& GetConverter() const { return m_conv; }
+#endif
 private :
     ATSUStyle m_macATSUIStyle;
+#if !wxUSE_UNICODE
+    wxCSConv m_conv;
+#endif
 };
 
 wxMacCoreGraphicsFontData::wxMacCoreGraphicsFontData(wxGraphicsRenderer* renderer, const wxFont &font, const wxColour& col) : wxGraphicsObjectRefData( renderer )
+#if !wxUSE_UNICODE
+    , m_conv( font.GetEncoding() == wxFONTENCODING_DEFAULT ? wxFONTENCODING_SYSTEM : font.GetEncoding() )
+#endif
 {
     m_macATSUIStyle = NULL;
 
@@ -759,6 +788,27 @@ wxMacCoreGraphicsFontData::~wxMacCoreGraphicsFontData()
     }
 }
 
+class wxMacCoreGraphicsBitmapData : public wxGraphicsObjectRefData
+{
+public:
+    wxMacCoreGraphicsBitmapData( wxGraphicsRenderer* renderer, const wxBitmap& bmp );
+    ~wxMacCoreGraphicsBitmapData();
+
+    virtual CGImageRef GetBitmap() { return m_bitmap; }
+private :
+    CGImageRef m_bitmap;
+};
+
+wxMacCoreGraphicsBitmapData::wxMacCoreGraphicsBitmapData( wxGraphicsRenderer* renderer, const wxBitmap& bmp ) : wxGraphicsObjectRefData( renderer )
+{
+    m_bitmap = (CGImageRef)( bmp.CGImageCreate() );
+}
+
+wxMacCoreGraphicsBitmapData::~wxMacCoreGraphicsBitmapData()
+{
+    CGImageRelease( m_bitmap );
+}
+
 //
 // Graphics Matrix
 //
@@ -1252,6 +1302,7 @@ public:
     //
     // image support
     //
+    void DrawGraphicsBitmapInternal( const wxGraphicsBitmap &bmp, wxDouble x, wxDouble y, wxDouble w, wxDouble h );
 
     virtual void DrawBitmap( const wxBitmap &bmp, wxDouble x, wxDouble y, wxDouble w, wxDouble h );
 
@@ -1291,14 +1342,31 @@ private:
 
 IMPLEMENT_DYNAMIC_CLASS(wxMacCoreGraphicsContext, wxGraphicsContext)
 
+class wxQuartzOffsetHelper
+{
+public :
+    wxQuartzOffsetHelper( CGContextRef cg , bool offset )
+    {
+        m_cg = cg;
+        m_offset = offset;
+        if ( m_offset )
+            CGContextTranslateCTM( m_cg, 0.5, 0.5 );
+    }
+    ~wxQuartzOffsetHelper( )
+    {
+        if ( m_offset )
+            CGContextTranslateCTM( m_cg, -0.5, -0.5 );
+    }
+public :
+    CGContextRef m_cg;
+    bool m_offset;
+} ;
+
 void wxMacCoreGraphicsContext::Init()
 {
     m_cgContext = NULL;
     m_releaseContext = false;
     m_windowRef = NULL;
-
-    HIRect r = CGRectMake(0,0,0,0);
-    m_clipRgn.Set(HIShapeCreateWithRect(&r));
 }
 
 wxMacCoreGraphicsContext::wxMacCoreGraphicsContext( wxGraphicsRenderer* renderer, CGContextRef cgcontext ) : wxGraphicsContext(renderer)
@@ -1359,15 +1427,24 @@ void wxMacCoreGraphicsContext::EnsureIsValid()
         CGContextConcatCTM( m_cgContext, m_windowTransform );
 		CGContextSaveGState( m_cgContext );
 		m_releaseContext = true;
-		if ( !HIShapeIsEmpty(m_clipRgn) )
+		if ( (HIShapeRef) m_clipRgn != NULL )
 		{
             // the clip region is in device coordinates, so we convert this again to user coordinates
             wxMacCFRefHolder<HIMutableShapeRef> hishape ;
             hishape.Set( HIShapeCreateMutableCopy( m_clipRgn ) );
             CGPoint transformedOrigin = CGPointApplyAffineTransform( CGPointZero,m_windowTransform);
             HIShapeOffset( hishape, -transformedOrigin.x, -transformedOrigin.y );
-			HIShapeReplacePathInCGContext( hishape, m_cgContext );
-			CGContextClip( m_cgContext );
+            // if the shape is empty, HIShapeReplacePathInCGContext doesn't work
+            if ( HIShapeIsEmpty(hishape))
+            {
+                CGRect empty = CGRectMake( 0,0,0,0 );
+                CGContextClipToRect( m_cgContext, empty );
+            }
+            else
+            {
+                HIShapeReplacePathInCGContext( hishape, m_cgContext );
+                CGContextClip( m_cgContext );
+            }
 		}
 		CGContextSaveGState( m_cgContext );
 	}
@@ -1415,10 +1492,19 @@ void wxMacCoreGraphicsContext::Clip( const wxRegion &region )
 {
     if( m_cgContext )
     {
-        HIShapeRef shape = HIShapeCreateWithQDRgn( (RgnHandle) region.GetWXHRGN() );
-        HIShapeReplacePathInCGContext( shape, m_cgContext );
-        CGContextClip( m_cgContext );
-        CFRelease( shape );
+        wxMacCFRefHolder<HIShapeRef> shape;
+        shape = HIShapeCreateWithQDRgn( (RgnHandle) region.GetWXHRGN() );
+        // if the shape is empty, HIShapeReplacePathInCGContext doesn't work
+        if ( HIShapeIsEmpty(shape))
+        {
+            CGRect empty = CGRectMake( 0,0,0,0 );
+            CGContextClipToRect( m_cgContext, empty );
+        }
+        else
+        {
+            HIShapeReplacePathInCGContext( shape, m_cgContext );
+            CGContextClip( m_cgContext );
+        }
     }
     else
     {
@@ -1468,8 +1554,7 @@ void wxMacCoreGraphicsContext::ResetClip()
     }
     else
     {
-        HIRect r = CGRectMake(0,0,0,0);
-        m_clipRgn.Set(HIShapeCreateWithRect(&r));
+        m_clipRgn.Release();
     }
 }
 
@@ -1480,16 +1565,11 @@ void wxMacCoreGraphicsContext::StrokePath( const wxGraphicsPath &path )
 
     EnsureIsValid();
 
-    bool offset = ShouldOffset();
-    if ( offset )
-        CGContextTranslateCTM( m_cgContext, 0.5, 0.5 );
+    wxQuartzOffsetHelper helper( m_cgContext , ShouldOffset() );
 
     ((wxMacCoreGraphicsPenData*)m_pen.GetRefData())->Apply(this);
     CGContextAddPath( m_cgContext , (CGPathRef) path.GetNativePath() );
     CGContextStrokePath( m_cgContext );
-
-    if ( offset )
-        CGContextTranslateCTM( m_cgContext, -0.5, -0.5 );
 }
 
 void wxMacCoreGraphicsContext::DrawPath( const wxGraphicsPath &path , int fillStyle )
@@ -1535,16 +1615,10 @@ void wxMacCoreGraphicsContext::DrawPath( const wxGraphicsPath &path , int fillSt
     if ( !m_pen.IsNull() )
         ((wxMacCoreGraphicsPenData*)m_pen.GetRefData())->Apply(this);
 
-    bool offset = ShouldOffset();
-
-    if ( offset )
-        CGContextTranslateCTM( m_cgContext, 0.5, 0.5 );
+    wxQuartzOffsetHelper helper( m_cgContext , ShouldOffset() );
 
     CGContextAddPath( m_cgContext , (CGPathRef) path.GetNativePath() );
     CGContextDrawPath( m_cgContext , mode );
-
-    if ( offset )
-        CGContextTranslateCTM( m_cgContext, -0.5, -0.5 );
 }
 
 void wxMacCoreGraphicsContext::FillPath( const wxGraphicsPath &path , int fillStyle )
@@ -1636,40 +1710,44 @@ void wxMacCoreGraphicsContext::Rotate( wxDouble angle )
         m_windowTransform = CGAffineTransformRotate(m_windowTransform,angle);
 }
 
+void wxGraphicsContext::DrawGraphicsBitmap( const wxGraphicsBitmap &bmp, wxDouble x, wxDouble y, wxDouble w, wxDouble h )
+{
+    static_cast<wxMacCoreGraphicsContext*>(this)->DrawGraphicsBitmapInternal(bmp, x, y, w, h);
+}
+
 void wxMacCoreGraphicsContext::DrawBitmap( const wxBitmap &bmp, wxDouble x, wxDouble y, wxDouble w, wxDouble h )
+{
+    wxGraphicsBitmap bitmap = GetRenderer()->CreateBitmap(bmp);
+    DrawGraphicsBitmapInternal(bitmap, x, y, w, h);
+}
+
+void wxMacCoreGraphicsContext::DrawGraphicsBitmapInternal( const wxGraphicsBitmap &bmp, wxDouble x, wxDouble y, wxDouble w, wxDouble h )
 {
     EnsureIsValid();
 
-    CGImageRef image = (CGImageRef)( bmp.CGImageCreate() );
+    CGImageRef image = static_cast<wxMacCoreGraphicsBitmapData*>(bmp.GetRefData())->GetBitmap();
     HIRect r = CGRectMake( x , y , w , h );
-    if ( bmp.GetDepth() == 1 )
+
+    // set the brush, which is used when the bitmap's depth == 1 
+    if (  !m_brush.IsNull() )
     {
-        // is is a mask, the '1' in the mask tell where to draw the current brush
-        if (  !m_brush.IsNull() )
+        if ( ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->IsShading() )
         {
-            if ( ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->IsShading() )
-            {
-                // TODO clip to mask
-            /*
-                CGContextSaveGState( m_cgContext );
-                CGContextAddPath( m_cgContext , (CGPathRef) path.GetNativePath() );
-                CGContextClip( m_cgContext );
-                CGContextDrawShading( m_cgContext, ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->GetShading() );
-                CGContextRestoreGState( m_cgContext);
-            */
-            }
-            else
-            {
-                ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->Apply(this);
-                HIViewDrawCGImage( m_cgContext , &r , image );
-            }
+            // TODO clip to mask
+        /*
+            CGContextSaveGState( m_cgContext );
+            CGContextAddPath( m_cgContext , (CGPathRef) path.GetNativePath() );
+            CGContextClip( m_cgContext );
+            CGContextDrawShading( m_cgContext, ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->GetShading() );
+            CGContextRestoreGState( m_cgContext);
+        */
+        }
+        else
+        {
+            ((wxMacCoreGraphicsBrushData*)m_brush.GetRefData())->Apply(this);
         }
     }
-    else
-    {
-        HIViewDrawCGImage( m_cgContext , &r , image );
-    }
-    CGImageRelease( image );
+    HIViewDrawCGImage( m_cgContext , &r , image );
 }
 
 void wxMacCoreGraphicsContext::DrawIcon( const wxIcon &icon, wxDouble x, wxDouble y, wxDouble w, wxDouble h )
@@ -1706,8 +1784,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 
 void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, wxDouble angle )
 {
-    if ( m_font.IsNull() )
-        return;
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::DrawText - no valid font set") );
 
     EnsureIsValid();
 
@@ -1723,7 +1800,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , str.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1733,7 +1810,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 #if wxUSE_UNICODE
     ubuf = (UniChar*) str.wc_str();
 #else
-    wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
@@ -1821,7 +1898,19 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *width, wxDouble *height,
                             wxDouble *descent, wxDouble *externalLeading ) const
 {
-    wxCHECK_RET( !m_font.IsNull(), wxT("wxDC(cg)::DoGetTextExtent - no valid font set") );
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::GetTextExtent - no valid font set") );
+
+    if ( width )
+        *width = 0;
+    if ( height )
+        *height = 0;
+    if ( descent )
+        *descent = 0;
+    if ( externalLeading )
+        *externalLeading = 0;
+        
+    if (str.empty())
+        return;
 
     OSStatus status = noErr;
 
@@ -1836,7 +1925,7 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , str.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1846,7 +1935,7 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
 #if wxUSE_UNICODE
     ubuf = (UniChar*) str.wc_str();
 #else
-    wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
@@ -1858,6 +1947,9 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
 
     wxASSERT_MSG( status == noErr , wxT("couldn't create the layout of the text") );
 
+    status = ::ATSUSetTransientFontMatching( atsuLayout , true );
+    wxASSERT_MSG( status == noErr , wxT("couldn't setup transient font matching") );
+
     ATSUTextMeasurement textBefore, textAfter;
     ATSUTextMeasurement textAscent, textDescent;
 
@@ -1884,12 +1976,15 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
     widths.Empty();
     widths.Add(0, text.length());
 
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::GetPartialTextExtents - no valid font set") );
+
     if (text.empty())
         return;
 
     ATSUTextLayout atsuLayout;
     UniCharCount chars = text.length();
     UniChar* ubuf = NULL;
+    OSStatus status = noErr;
 
 #if SIZEOF_WCHAR_T == 4
     wxMBConvUTF16 converter;
@@ -1898,7 +1993,7 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , text.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = text.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = text.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1908,16 +2003,22 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
 #if wxUSE_UNICODE
     ubuf = (UniChar*) text.wc_str();
 #else
-    wxWCharBuffer wchar = text.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = text.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
 #endif
 
     ATSUStyle style = (((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetATSUStyle());
-    ::ATSUCreateTextLayoutWithTextPtr( (UniCharArrayPtr) ubuf , 0 , chars , chars , 1 ,
+    status = ::ATSUCreateTextLayoutWithTextPtr( (UniCharArrayPtr) ubuf , 0 , chars , chars , 1 ,
         &chars , &style , &atsuLayout );
+    wxASSERT_MSG( status == noErr , wxT("couldn't create the layout of the text") );
 
+    status = ::ATSUSetTransientFontMatching( atsuLayout , true );
+    wxASSERT_MSG( status == noErr , wxT("couldn't setup transient font matching") );
+
+// new implementation from JS, keep old one just in case
+#if 0
     for ( int pos = 0; pos < (int)chars; pos ++ )
     {
         unsigned long actualNumberOfBounds = 0;
@@ -1933,6 +2034,31 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
         widths[pos] = FixedToInt( glyphBounds.upperRight.x - glyphBounds.upperLeft.x );
         //unsigned char uch = s[i];
     }
+#else
+    ATSLayoutRecord *layoutRecords = NULL;
+    ItemCount glyphCount = 0;
+    
+    // Get the glyph extents
+    OSStatus err = ::ATSUDirectGetLayoutDataArrayPtrFromTextLayout(atsuLayout,
+                                                                   0,
+                                                                   kATSUDirectDataLayoutRecordATSLayoutRecordCurrent,
+                                                                   (void **)
+                                                                   &layoutRecords,
+                                                                   &glyphCount);
+    wxASSERT(glyphCount == (text.length()+1));
+    
+    if ( err == noErr && glyphCount == (text.length()+1))
+    {
+        for ( int pos = 1; pos < (int)glyphCount ; pos ++ )
+        {
+            widths[pos-1] = FixedToInt( layoutRecords[pos].realPos );
+        }
+    }
+    
+    ::ATSUDirectReleaseLayoutDataArrayPtr(NULL,
+                                          kATSUDirectDataLayoutRecordATSLayoutRecordCurrent,
+                                          (void **) &layoutRecords);
+#endif
 
     ::ATSUDisposeTextLayout(atsuLayout);
 #if SIZEOF_WCHAR_T == 4
@@ -2031,6 +2157,8 @@ public :
 
    // sets the font
     virtual wxGraphicsFont CreateFont( const wxFont &font , const wxColour &col = *wxBLACK ) ;
+    
+    wxGraphicsBitmap CreateBitmap( const wxBitmap &bitmap ) ;
 
 private :
     DECLARE_DYNAMIC_CLASS_NO_COPY(wxMacCoreGraphicsRenderer)
@@ -2166,6 +2294,18 @@ wxGraphicsFont wxMacCoreGraphicsRenderer::CreateFont( const wxFont &font , const
         return wxNullGraphicsFont;
 }
 
+wxGraphicsBitmap wxGraphicsRenderer::CreateBitmap( const wxBitmap& bmp )
+{
+    if ( bmp.Ok() )
+    {
+        wxGraphicsBitmap p;
+        p.SetRefData(new wxMacCoreGraphicsBitmapData( this , bmp ));
+        return p;
+    }
+    else
+        return wxNullGraphicsBitmap;
+}
+
 
 
 #endif // wxMAC_USE_CORE_GRAPHICS
