@@ -6,7 +6,7 @@
 # Author:       Robin Dunn
 #
 # Created:      A long time ago, in a galaxy far, far away...
-# RCS-ID:       $Id: Main.py 58881 2009-02-13 14:47:34Z AG $
+# RCS-ID:       $Id: Main.py 59671 2009-03-20 20:58:50Z RD $
 # Copyright:    (c) 1999 by Total Control Software
 # Licence:      wxWindows license
 #----------------------------------------------------------------------------
@@ -112,6 +112,8 @@ _treeList = [
         'ResizeWidget',
         'Cairo',
         'Cairo_Snippets',
+        'SystemSettings',
+        'GridLabelRenderer',
         ]),
 
     # managed windows == things with a (optional) caption you can close
@@ -159,6 +161,7 @@ _treeList = [
         'Gauge',
         'Grid',
         'Grid_MegaExample',
+        'GridLabelRenderer',
         'ListBox',
         'ListCtrl',
         'ListCtrl_virtual',
@@ -334,6 +337,7 @@ _treeList = [
         'ShapedWindow',
         'Sound',
         'StandardPaths',
+        'SystemSettings',
         'Unicode',
         ]),
 
