@@ -62,12 +62,32 @@ PyCollapsiblePane is freeware and distributed under the wxPython license.
 Latest Revision: Andrea Gavana @ 15 Oct 2008, 22.30 GMT
 Version 0.1
 
+Latest Revision: Robin Dunn @ 10-Apr-2009
+Version 0.2
+
 """
 
 import wx
 
 CP_GTK_EXPANDER = 4
+CP_USE_STATICBOX = 8
+CP_LINE_ABOVE = 16
 
+# inject into the wx namespace with the other CP_* constants
+wx.CP_GTK_EXPANDER = CP_GTK_EXPANDER
+wx.CP_USE_STATICBOX = CP_USE_STATICBOX
+wx.CP_LINE_ABOVE = CP_LINE_ABOVE
+
+#-----------------------------------------------------------------------------
+# 2.9 deprecates SetSpacer, so we'll use AssignSpacer and monkey-patch 2.8 so
+# it works there too
+
+if wx.VERSION < (2, 9):
+    wx.SizerItem.AssignSpacer = wx.SizerItem.SetSpacer
+
+#-----------------------------------------------------------------------------
+# GTKExpander widget
+#-----------------------------------------------------------------------------
 
 class GTKExpander(wx.PyControl):
 
@@ -142,11 +162,11 @@ class GTKExpander(wx.PyControl):
 # PyCollapsiblePane
 #-----------------------------------------------------------------------------
 
-class PyCollapsiblePane(wx.PyControl):
+class PyCollapsiblePane(wx.PyPanel):
 
     def __init__(self, parent, id=wx.ID_ANY, label="", pos=wx.DefaultPosition,
-                 size=wx.DefaultSize, style=wx.CP_DEFAULT_STYLE, val=wx.DefaultValidator,
-                 name="CollapsiblePane"):
+                 size=wx.DefaultSize, style=wx.CP_DEFAULT_STYLE, 
+                 validator=wx.DefaultValidator, name="CollapsiblePane"):
         """
         Default class constructor.
 
@@ -157,76 +177,85 @@ class PyCollapsiblePane(wx.PyControl):
         @param pos: Initial position.
         @param size: Initial size.
         @param style: The window style, see wx.CP_* flags.
-        @param validator: Validator which can be used for additional data checks.
+        @param validator: (not used).
         @param name: Control name.
         """
         
-        wx.PyControl.__init__(self, parent, id, pos, size, style, val, name)
+        wx.PyPanel.__init__(self, parent, id, pos, size, style, name)
         
         self._pButton = self._pStaticLine = self._pPane = self._sz = None            
         self._strLabel = label
         self._bCollapsed = True
 
-        if style & CP_GTK_EXPANDER == 0:
-            # create children and lay them out using a wx.BoxSizer
-            # (so that we automatically get RTL features)
-            self._pButton = wx.Button(self, wx.ID_ANY, self.GetBtnLabel(), wx.Point(0, 0),
-                                      wx.DefaultSize, wx.BU_EXACTFIT)
-            self._pStaticLine = wx.StaticLine(self, wx.ID_ANY)
+        self._pPane = wx.Panel(self, style=wx.TAB_TRAVERSAL|wx.NO_BORDER)
+        self._pPane.Hide()
 
-            if wx.Platform == "__WXMAC__": 
-                # on Mac we put the static libe above the button
-                self._sz = wx.BoxSizer(wx.VERTICAL)
-                self._sz.Add(self._pStaticLine, 0, wx.ALL|wx.GROW, self.GetBorder())
-                self._sz.Add(self._pButton, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, self.GetBorder())
-            else:
-                # on other platforms we put the static line and the button horizontally
-                self._sz = wx.BoxSizer(wx.HORIZONTAL)
-                self._sz.Add(self._pButton, 0, wx.LEFT|wx.TOP|wx.BOTTOM, self.GetBorder())
-                self._sz.Add(self._pStaticLine, 1, wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT, self.GetBorder())
-        else:
+        if self.HasFlag(CP_USE_STATICBOX):
+            # Use a StaticBox instead of a StaticLine, and the button's
+            # position will be handled separately so don't put it in the sizer
+            self._pStaticBox = wx.StaticBox(self)
+            self.SetButton(wx.Button(self, wx.ID_ANY, self.GetLabel(), style=wx.BU_EXACTFIT))
+            self._sz = wx.BoxSizer(wx.VERTICAL)
+            self._sz.Add((1,1))  # spacer, size will be reset later
+            self._contentSizer = wx.StaticBoxSizer(self._pStaticBox, wx.VERTICAL)
+            self._contentSizer.Add((1,1))  # spacer, size will be reset later
+            self._contentSizer.Add(self._pPane, 1, wx.EXPAND)
+            self._sz.Add(self._contentSizer, 1, wx.EXPAND)                    
+
+            if self.HasFlag(CP_USE_STATICBOX) and 'wxMSW' in wx.PlatformInfo:
+                # This hack is needed on Windows because wxMSW clears the
+                # CLIP_SIBLINGS style from all sibling controls that overlap the
+                # static box, so the box ends up overdrawing the button since we
+                # have the button overlapping the box. This hack will ensure that
+                # the button is refreshed after every time that the box is drawn.
+                # This adds a little flicker but it is not too bad compared to
+                # others.
+                def paint(evt):
+                    def updateBtn():
+                        self._pButton.Refresh()
+                        self._pButton.Update()
+                    wx.CallAfter(updateBtn)
+                    evt.Skip()
+                self._pStaticBox.Bind(wx.EVT_PAINT, paint)
+
+        elif self.HasFlag(CP_GTK_EXPANDER):
             self._sz = wx.BoxSizer(wx.HORIZONTAL)
-            self._pButton = GTKExpander(self, wx.ID_ANY, self.GetBtnLabel(), wx.Point(0, 0),
-                                        wx.DefaultSize)
-            self._sz.Add(self._pButton, 0, wx.LEFT|wx.TOP|wx.BOTTOM, self.GetBorder())
             self.SetExpanderDimensions(3, 6)
+            self.SetButton(GTKExpander(self, wx.ID_ANY, self.GetLabel()))
+            self._sz.Add(self._pButton, 0, wx.LEFT|wx.TOP|wx.BOTTOM, self.GetBorder())
 
             self._pButton.Bind(wx.EVT_PAINT, self.OnDrawGTKStyle)
             self._pButton.Bind(wx.EVT_LEFT_DOWN, self.OnButton)
             if wx.Platform == "__WXMSW__":
                 self._pButton.Bind(wx.EVT_LEFT_DCLICK, self.OnButton)
+
+        else:
+            # create children and lay them out using a wx.BoxSizer
+            # (so that we automatically get RTL features)
+            self.SetButton(wx.Button(self, wx.ID_ANY, self.GetLabel(), style=wx.BU_EXACTFIT))
+            self._pStaticLine = wx.StaticLine(self, wx.ID_ANY)
+
+            if self.HasFlag(CP_LINE_ABOVE): 
+                # put the static libe above the button
+                self._sz = wx.BoxSizer(wx.VERTICAL)
+                self._sz.Add(self._pStaticLine, 0, wx.ALL|wx.GROW, self.GetBorder())
+                self._sz.Add(self._pButton, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, self.GetBorder())
+            else:
+                # arrainge the static line and the button horizontally
+                self._sz = wx.BoxSizer(wx.HORIZONTAL)
+                self._sz.Add(self._pButton, 0, wx.LEFT|wx.TOP|wx.BOTTOM, self.GetBorder())
+                self._sz.Add(self._pStaticLine, 1, wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT, self.GetBorder())
             
-        # do not set sz as our sizers since we handle the pane window without using sizers
-        self._pPane = wx.Panel(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
-                               wx.TAB_TRAVERSAL|wx.NO_BORDER)
-
-        # start as collapsed
-        self._pPane.Hide()
-
-        self.Bind(wx.EVT_BUTTON, self.OnButton, self._pButton)
         self.Bind(wx.EVT_SIZE, self.OnSize)
         
 
-    def DoGetBestSize(self):
-
-        # NB: do not use GetSize() but rather GetMinSize()
-        sz = self._sz.GetMinSize()
-
-        # when expanded, we need more vertical space
-        if self.IsExpanded():
-            sz.SetWidth(max(sz.GetWidth(), self._pPane.GetBestSize().x))
-            sz.SetHeight(sz.y + self.GetBorder() + self._pPane.GetBestSize().y)
-        
-        return sz
-
-
     def GetBtnLabel(self):
         """ Returns the button label. """
 
         if self.GetWindowStyleFlag() & CP_GTK_EXPANDER:
-            return self._strLabel
+            return self.GetLabel()
         
-        return self._strLabel + (self.IsCollapsed() and [" >>"] or [" <<"])[0]
+        return self.GetLabel() + (self.IsCollapsed() and [" >>"] or [" <<"])[0]
 
 
     def OnStateChange(self, sz):
@@ -340,55 +369,16 @@ class PyCollapsiblePane(wx.PyControl):
     def SetLabel(self, label):
         """ Sets the button label. """
 
-        self._strLabel = label
+        self._strLabel =  label
         self._pButton.SetLabel(self.GetBtnLabel())
         self._pButton.SetInitialSize()
         self._pButton.Refresh()
 
         self.Layout()
 
-
-    def SetButton(self, button):
-        """
-        Assign a new button to PyCollapsiblePane.
-        This button can be the standard wx.Button or any of the generic
-        implementations which live in wx.lib.buttons.
-        """
-
-        self._sz.Replace(self._pButton, button)
-        self._pButton.Destroy()
-
-        self._pButton = button
-        self.SetLabel(button.GetLabel())
-        self.Bind(wx.EVT_BUTTON, self.OnButton, self._pButton)
-        
-        self.Layout()
-        
-
-    def Layout(self):
-        """ Layout the PyCollapsiblePane. """
-
-        if not self._pButton or not self._pPane or not self._sz:
-            return False     # we need to complete the creation first!
-
-        oursz = self.GetSize()
-
-        # move & resize the button and the static line
-        self._sz.SetDimension(0, 0, oursz.GetWidth(), self._sz.GetMinSize().GetHeight())
-        self._sz.Layout()
-
-        if self.IsExpanded():
-        
-            # move & resize the container window
-            yoffset = self._sz.GetSize().GetHeight() + self.GetBorder()
-            self._pPane.SetDimensions(0, yoffset, oursz.x, oursz.y - yoffset)
-
-            # this is very important to make the pane window layout show correctly
-            self._pPane.Show()
-            self._pPane.Layout()
-        
-        return True
-
+    def GetLabel(self):
+        return self._strLabel
+    
 
     def GetBorder(self):
         """ Returns the PyCollapsiblePane border (platform dependent). """
@@ -406,14 +396,121 @@ class PyCollapsiblePane(wx.PyControl):
     def SetExpanderDimensions(self, width, height):
 
         self._expanderDimensions = width, height
-        self._sz.Layout()
-        self._pButton.Refresh()
+        if self._sz:
+            self._sz.Layout()
+        if self._pButton:
+            self._pButton.Refresh()
 
 
     def GetExpanderDimensions(self):
 
         return self._expanderDimensions
+    
+        
+    def DoGetBestSize(self):
+        if self.HasFlag(CP_USE_STATICBOX):
+            # In this case the button is not in the sizer, and the static box
+            # is not shown when not expanded, so use the size of the button as
+            # our stating point. But first we'll make sure that it is it's
+            # best size.
+            sz = self._pButton.GetBestSize()
+            self._pButton.SetSize(sz)
+            bdr = self.GetBorder()
+            sz += (2*bdr, 2*bdr)
+                        
+            if self.IsExpanded():
+                # Reset the spacer sizes to be based on the button size
+                adjustment = 0
+                if 'wxMac' not in wx.PlatformInfo:
+                    adjustment = -3
+                self._sz.Children[0].AssignSpacer((1, sz.height/2 + adjustment))
+                self._contentSizer.Children[0].AssignSpacer((1, sz.height/2))
 
+                ssz = self._sz.GetMinSize()
+                sz.width = max(sz.width, ssz.width)
+                sz.height = max(sz.height, ssz.height)
+                
+        else:
+            # do not use GetSize() but rather GetMinSize() since it calculates
+            # the required space of the sizer
+            sz = self._sz.GetMinSize()
+
+            # when expanded, we need more space
+            if self.IsExpanded():
+                pbs = self._pPane.GetBestSize()
+                sz.width = max(sz.GetWidth(), pbs.x)
+                sz.height = sz.y + self.GetBorder() + pbs.y
+        
+        return sz
+
+    
+    def Layout(self):
+        """ Layout the PyCollapsiblePane. """
+
+        if not self._pButton or not self._pPane or not self._sz:
+            return False     # we need to complete the creation first!
+
+        oursz = self.GetSize()
+
+        if self.HasFlag(CP_USE_STATICBOX):
+            bdr = self.GetBorder()
+            self._pButton.SetPosition((bdr, bdr))
+            
+            if self.IsExpanded():
+                self._pPane.Show()
+                self._pPane.Layout()
+                self._pStaticBox.Show()
+                self._pStaticBox.Lower()
+                self._pButton.Raise()
+                self._sz.SetDimension(0, 0, oursz.width, oursz.height)
+                self._sz.Layout()
+            else:
+                if hasattr(self, '_pStaticBox'):
+                    self._pStaticBox.Hide()
+                
+        else:    
+            # move & resize the button and the static line
+            self._sz.SetDimension(0, 0, oursz.GetWidth(), self._sz.GetMinSize().GetHeight())
+            self._sz.Layout()
+    
+            if self.IsExpanded():
+            
+                # move & resize the container window
+                yoffset = self._sz.GetSize().GetHeight() + self.GetBorder()
+                self._pPane.SetDimensions(0, yoffset, oursz.x, oursz.y - yoffset)
+    
+                # this is very important to make the pane window layout show correctly
+                self._pPane.Show()
+                self._pPane.Layout()
+        
+        return True
+
+    
+    def SetButton(self, button):
+        """
+        Assign a new button to PyCollapsiblePane.
+        This button can be the standard wx.Button or any of the generic
+        implementations which live in wx.lib.buttons.
+        """
+        if self._pButton:
+            if not self.HasFlag(CP_USE_STATICBOX):
+                self._sz.Replace(self._pButton, button)
+            self.Unbind(wx.EVT_BUTTON, self._pButton)
+            self._pButton.Destroy()
+
+        self._pButton = button
+        self.SetLabel(button.GetLabel())
+        self.Bind(wx.EVT_BUTTON, self.OnButton, self._pButton)
+
+        if self._pPane:
+            self._pButton.MoveBeforeInTabOrder(self._pPane)
+        self.Layout()
+        
+            
+    def GetButton(self):
+        return self._pButton
+    
+    
     
 #-----------------------------------------------------------------------------
 # PyCollapsiblePane - event handlers
@@ -461,3 +558,8 @@ class PyCollapsiblePane(wx.PyControl):
 
         self._pButton.OnDrawGTKText(dc)
 
+
+        
+        
+        
+        
