@@ -35,21 +35,28 @@ eclib.ctrlbox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: finddlg.py 58357 2009-01-24 17:53:43Z CJP $"
-__revision__ = "$Revision: 58357 $"
+__svnid__ = "$Id: finddlg.py 60131 2009-04-14 02:44:17Z CJP $"
+__revision__ = "$Revision: 60131 $"
 
 __all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
-           "MiniFindReplaceDialog", "AdvFindReplaceDlg",
+           "MiniFindReplaceDlg", "AdvFindReplaceDlg",
+
            "AFR_STYLE_FINDDIALOG", "AFR_STYLE_REPLACEDIALOG",
-           "AFR_STYLE_NON_FLOATING", "AFR_UP", "AFR_WHOLEWORD", 
+           "AFR_STYLE_NON_FLOATING",
+           
+           "AFR_UP", "AFR_WHOLEWORD", 
            "AFR_MATCHCASE", "AFR_REGEX", "AFR_RECURSIVE", "AFR_NOLOOKIN",
            "AFR_NOUPDOWN", "AFR_NOWHOLEWORD", "AFR_NOMATCHCASE", "AFR_NOREGEX",
-           "AFR_NOFILTER", "AFR_NOOPTIONS", "LOCATION_CURRENT_DOC", 
-           "LOCATION_OPEN_DOCS", "LOCATION_IN_FILES", "LOCATION_MAX", 
+           "AFR_NOFILTER", "AFR_NOOPTIONS",
+
+           "LOCATION_CURRENT_DOC",
+           "LOCATION_OPEN_DOCS", "LOCATION_IN_FILES", "LOCATION_MAX",
+
            "edEVT_FIND_CLOSE", "EVT_FIND_CLOSE", "edEVT_FIND", "EVT_FIND",
            "edEVT_FIND_NEXT", "EVT_FIND_NEXT", "edEVT_FIND_ALL", "EVT_FIND_ALL",
            "edEVT_REPLACE", "EVT_REPLACE", "edEVT_REPLACE_ALL",
-           "EVT_REPLACE_ALL"]
+           "EVT_REPLACE_ALL", "EVT_OPTION_CHANGED", "edEVT_OPTION_CHANGED",
+           "EVT_COUNT", "edEVT_COUNT"]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -100,6 +107,7 @@ ID_MATCH_CASE = wx.NewId()
 ID_WHOLE_WORD = wx.NewId()
 ID_REGEX = wx.NewId()
 ID_RECURSE = wx.NewId()
+ID_COUNT = wx.NewId()
 ID_FIND_ALL = wx.NewId()
 ID_REPLACE_ALL = wx.NewId()
 ID_OPTION_CHANGE = wx.NewId()
@@ -131,6 +139,10 @@ EVT_FIND_ALL = wx.PyEventBinder(edEVT_FIND_ALL, 1)
 edEVT_REPLACE = wx.NewEventType()
 EVT_REPLACE = wx.PyEventBinder(edEVT_REPLACE, 1)
 
+# Replace button clicked
+edEVT_COUNT = wx.NewEventType()
+EVT_COUNT = wx.PyEventBinder(edEVT_COUNT, 1)
+
 # Replace All button clicked
 edEVT_REPLACE_ALL = wx.NewEventType()
 EVT_REPLACE_ALL = wx.PyEventBinder(edEVT_REPLACE_ALL, 1)
@@ -142,6 +154,7 @@ EVT_OPTION_CHANGED = wx.PyEventBinder(edEVT_OPTION_CHANGED, 1)
 # Convenience for generating events
 _EVENT_MAP = { wx.ID_FIND : edEVT_FIND,
                wx.ID_REPLACE : edEVT_REPLACE,
+               ID_COUNT : edEVT_COUNT,
                ID_FIND_ALL : edEVT_FIND_ALL,
                ID_REPLACE_ALL : edEVT_REPLACE_ALL,
                ID_OPTION_CHANGE : edEVT_OPTION_CHANGED }
@@ -166,6 +179,14 @@ class FindEvent(wx.PyCommandEvent):
         self._replace = u''
         self._dir = u''
         self._filters = None
+        self._count = 0
+
+    def GetCount(self):
+        """Get the number of matches
+        @return: int
+
+        """
+        return self._count
 
     def GetDirectory(self):
         """Get the directory of files to search in
@@ -209,6 +230,13 @@ class FindEvent(wx.PyCommandEvent):
         """
         return self._loc
 
+    def SetCount(self, count):
+        """Set the count
+        @param count: int
+
+        """
+        self._count = count
+
     def SetDirectory(self, directory):
         """Set the directory of files to search in
         @param directory: string
@@ -314,7 +342,8 @@ def AdvFindReplaceDlg(parent, fdata, title, style=AFR_STYLE_FINDDIALOG):
 #--------------------------------------------------------------------------#
 
 class FindReplaceDlgBase:
-    """Mixin Base class for deriving FindReplaceDialogs.
+    """Delegate mixin base class for deriving FindReplaceDialogs, delegates
+    calls to the widgets and containers that make up the dialog.
     @note: The mixin must be initialized after the class its being mixed into
 
     """
@@ -452,7 +481,7 @@ class FindReplaceDlgBase:
         """
         self._box.SetFindBitmap(bmp)
 
-    def SetFlag(self, flag):
+    def SetFlag(self, flags):
         """Set a search dialog flag.
         @param flags: AFR_*
 
@@ -473,6 +502,15 @@ class FindReplaceDlgBase:
         """
         self._panel.SetLookinChoices(paths)
 
+    def SetLookinPath(self, path):
+        """Set the lookin path, adding it to the collection if it is
+        not in there.
+        @param path: string (path of directory)
+
+        """
+        idx = self._panel.AddLookinPath(path)
+        self.SetLookinSelection(idx)
+
     def SetLookinSelection(self, sel):
         """Set the looking choices selection
         @param sel: int
@@ -503,7 +541,7 @@ class FindReplaceDlgBase:
 
     def SetReplaceString(self, rstring):
         """Set the replace controls search string
-        @param query: string
+        @param rstring: string
 
         """
         self._panel.SetReplaceString(rstring)
@@ -680,7 +718,8 @@ class FindPanel(wx.Panel):
         self.Bind(wx.EVT_CHECKBOX, self.OnOption)
         self.Bind(wx.EVT_RADIOBUTTON, self.OnOption)
         self.Bind(wx.EVT_CHOICE, self.OnChoice, id=ID_LOOKIN)
-        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_FIND_ALL, ID_REPLACE_ALL):
+        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_COUNT,
+                    ID_FIND_ALL, ID_REPLACE_ALL):
             self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI, id=bid)
         self.Bind(wx.EVT_SET_FOCUS, lambda evt: self.__SetFocus())
         self._ftxt.Bind(wx.EVT_SET_FOCUS, lambda evt: self._ftxt.SelectAll())
@@ -731,6 +770,7 @@ class FindPanel(wx.Panel):
         self._sizers['look'] = wx.BoxSizer(wx.VERTICAL)
         li_sz = wx.BoxSizer(wx.HORIZONTAL)
         dirbtn = wx.Button(self, ID_CHOOSE_DIR, u"...", style=wx.BU_EXACTFIT)
+        dirbtn.SetToolTipString(_("Choose Folder"))
         li_sz.AddMany([(self._lookin, 1, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                        (dirbtn, 0, wx.ALIGN_CENTER_VERTICAL)])
         li_lbl = wx.StaticText(self, label=_("Look in") + u":")
@@ -785,6 +825,7 @@ class FindPanel(wx.Panel):
         self._sizers['frspacer'] = bsizer.Add((50, 1), 1)
         for bid, blbl in [(wx.ID_FIND, _("Find")),
                           (wx.ID_REPLACE, _("Replace")),
+                          (ID_COUNT, _("Count")),
                           (ID_FIND_ALL, _("Find All")),
                           (ID_REPLACE_ALL, _("Replace All"))]:
             self._sizers[bid] = wx.BoxSizer(wx.HORIZONTAL)
@@ -839,11 +880,11 @@ class FindPanel(wx.Panel):
 
         """
         if find:
-            show = (wx.ID_FIND, ID_FIND_ALL, 'fspacer')
+            show = (wx.ID_FIND, ID_COUNT, ID_FIND_ALL, 'fspacer')
             hide = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
         else:
             show = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
-            hide = (ID_FIND_ALL, 'fspacer')
+            hide = (ID_FIND_ALL, ID_COUNT, 'fspacer')
 
         for ctrl in show:
             if isinstance(ctrl, basestring):
@@ -1070,7 +1111,8 @@ class FindPanel(wx.Panel):
         @param evt: wx.UpdateUIEvent
 
         """
-        if evt.GetId() in (wx.ID_FIND, wx.ID_REPLACE) and \
+        # TODO: support count in files?
+        if evt.GetId() in (wx.ID_FIND, ID_COUNT, wx.ID_REPLACE) and \
            self._lookin.GetSelection() > LOCATION_CURRENT_DOC:
             evt.Enable(False)
         else:
