@@ -14,8 +14,8 @@ for managing keybindings and profiles is also provided by this module.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_menu.py 58836 2009-02-12 03:35:25Z CJP $"
-__revision__ = "$Revision: 58836 $"
+__svnid__ = "$Id: ed_menu.py 60538 2009-05-07 02:56:45Z CJP $"
+__revision__ = "$Revision: 60538 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -66,7 +66,7 @@ class EdMenu(wx.Menu):
         """
         if use_bmp and item.GetKind() == wx.ITEM_NORMAL:
             self.SetItemBitmap(item)
-        wx.Menu.AppendItem(self, item)
+        super(EdMenu, self).AppendItem(item)
 
     def Insert(self, pos, id_, text=u'', helpstr=u'', \
                kind=wx.ITEM_NORMAL, use_bmp=True):
@@ -76,7 +76,7 @@ class EdMenu(wx.Menu):
                           available in the ArtProvider
 
         """
-        item = wx.Menu.Insert(self, pos, id_, text, helpstr, kind)
+        item = super(EdMenu, self).Insert(pos, id_, text, helpstr, kind)
         if use_bmp and kind == wx.ITEM_NORMAL:
             self.SetItemBitmap(item)
         return item
@@ -189,12 +189,9 @@ class EdMenu(wx.Menu):
         @param item: item to set bitmap for
 
         """
-        try:
-            bmp = wx.ArtProvider.GetBitmap(str(item.GetId()), wx.ART_MENU)
-            if not bmp.IsNull():
-                item.SetBitmap(bmp)
-        finally:
-            pass
+        bmp = wx.ArtProvider.GetBitmap(str(item.GetId()), wx.ART_MENU)
+        if not bmp.IsNull():
+            item.SetBitmap(bmp)
 
 #-----------------------------------------------------------------------------#
 
@@ -250,12 +247,20 @@ class KeyBinder(object):
                                      suffix='.ekeys', title=False)
         if recs == -1:
             recs = list()
+
+        tmp = util.GetResourceFiles(u'ekeys', True, True, '.ekeys', False)
+        if tmp != -1:
+            recs.extend(tmp)
+
         return recs
 
     def GetProfilePath(self, pname):
         """Get the full path to the given keyprofile
         @param pname: profile name
         @return: string or None
+        @note: expects unique name for each profile in the case that
+               a name exists in both the user and system paths the one
+               found on the user path will be returned.
 
         """
         if pname is None:
@@ -271,7 +276,12 @@ class KeyBinder(object):
         if rname is None:
             rname = pname
 
-        rname = u"%s%s.ekeys" % (ed_glob.CONFIG['CACHE_DIR'], rname)
+        kprof = u"%s%s.ekeys" % (ed_glob.CONFIG['CACHE_DIR'], rname)
+        if not os.path.exists(kprof):
+            # Must be a system supplied keyprofile
+            rname = u"%s%s.ekeys" % (ed_glob.CONFIG['KEYPROF_DIR'], rname)
+        else:
+            rname = kprof
 
         return rname
 
@@ -499,6 +509,8 @@ class EdMenuBar(wx.MenuBar):
                         u"..." + EdMenuBar.keybinder.GetBinding(ed_glob.ID_RELOAD_ENC),
                         _("Reload the file with a specified encoding"))
         filemenu.AppendSeparator()
+
+        # Profile
         pmenu = EdMenu()
         pmenu.Append(ed_glob.ID_SAVE_PROFILE, _("Save Profile") + \
                      EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVE_PROFILE),
@@ -508,6 +520,18 @@ class EdMenuBar(wx.MenuBar):
                      _("Load a Custom Profile"))
         filemenu.AppendSubMenu(pmenu, _("Profile"),
                                _("Load and save custom Profiles"))
+
+        # Sessions
+        smenu = EdMenu()
+        smenu.Append(ed_glob.ID_SAVE_SESSION, _("Save Session") + \
+                     EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVE_SESSION),
+                     _("Save the current session."))
+        smenu.Append(ed_glob.ID_LOAD_SESSION, _("Load Session") + \
+                     EdMenuBar.keybinder.GetBinding(ed_glob.ID_LOAD_SESSION),
+                     _("Load a saved session."))
+        filemenu.AppendSubMenu(smenu, _("Sessions"),
+                               _("Load and save custom sessions."))
+
         filemenu.AppendSeparator()
         filemenu.Append(ed_glob.ID_PRINT_SU, _("Page Set&up") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_PRINT_SU),
@@ -572,6 +596,9 @@ class EdMenuBar(wx.MenuBar):
         linemenu.Append(ed_glob.ID_CUT_LINE, _("Cut Line") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_CUT_LINE),
                         _("Cut Current Line"))
+        linemenu.Append(ed_glob.ID_DELETE_LINE, _("Delete Line") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_DELETE_LINE),
+                        _("Delete the selected line(s)"))
         linemenu.Append(ed_glob.ID_COPY_LINE, _("Copy Line") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_COPY_LINE),
                         _("Copy Current Line"))
@@ -586,6 +613,13 @@ class EdMenuBar(wx.MenuBar):
         linemenu.Append(ed_glob.ID_TRANSPOSE, _("Transpose Line") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_TRANSPOSE),
                         _("Transpose the current line with the previous one"))
+        linemenu.Append(ed_glob.ID_LINE_MOVE_UP, _("Move Current Line Up") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_LINE_MOVE_UP),
+                        _("Move the current line up"))
+        linemenu.Append(ed_glob.ID_LINE_MOVE_DOWN,
+                        _("Move Current Line Down") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_LINE_MOVE_DOWN),
+                        _("Move the current line down"))
         editmenu.AppendMenu(ed_glob.ID_LINE_EDIT, _("Line Edit"), linemenu,
                             _("Commands that affect an entire line"))
         bookmenu = EdMenu()
@@ -722,7 +756,6 @@ class EdMenuBar(wx.MenuBar):
                           _("Toggle comment on the selected line(s)"))
         formatmenu.AppendSeparator()
 
-
         formatmenu.Append(ed_glob.ID_INDENT, _("Indent Lines") + \
                           EdMenuBar.keybinder.GetBinding(ed_glob.ID_INDENT),
                           _("Indent the selected lines"))
@@ -818,10 +851,6 @@ class EdMenuBar(wx.MenuBar):
         toolsmenu.Append(ed_glob.ID_COMMAND, _("Editor Command") + \
                          EdMenuBar.keybinder.GetBinding(ed_glob.ID_COMMAND),
                          _("Goto command buffer"))
-        toolsmenu.Append(ed_glob.ID_KWHELPER, _("Keyword Helper") + \
-                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_KWHELPER),
-                         _("Provides a Contextual Help Menu Listing Standard "
-                           "Keywords/Functions"))
         toolsmenu.Append(ed_glob.ID_PLUGMGR, _("Plugin Manager") + \
                          EdMenuBar.keybinder.GetBinding(ed_glob.ID_PLUGMGR),
                          _("Manage, Download, and Install plugins"))
@@ -1006,10 +1035,13 @@ _DEFAULT_BINDING = { # File Menu
                      ed_glob.ID_LINE_AFTER : (u"Ctrl", u"L"),
                      ed_glob.ID_LINE_BEFORE : (u"Ctrl", u"Shift", u"L"),
                      ed_glob.ID_CUT_LINE : (u"Ctrl", u"D"),
+                     ed_glob.ID_DELETE_LINE : (u"Ctrl", u"Shift", "D"),
                      ed_glob.ID_COPY_LINE : (u"Ctrl", u"Y"),
                      ed_glob.ID_DUP_LINE : (u"Ctrl", u"Shift", u"C"),
                      ed_glob.ID_JOIN_LINES : (u"Ctrl", u"J"),
                      ed_glob.ID_TRANSPOSE : (u"Ctrl", u"T"),
+                     ed_glob.ID_LINE_MOVE_UP : (u"Ctrl", u"Shift", u"Up"),
+                     ed_glob.ID_LINE_MOVE_DOWN : (u"Ctrl", u"Shift", u"Down"),
                      ed_glob.ID_ADD_BM : (u"Ctrl", u"B"),
                      ed_glob.ID_FIND : (u"Ctrl", u"Shift", u"F"),
                      ed_glob.ID_FIND_PREVIOUS : (u"Shift", u"F3"),
@@ -1029,7 +1061,7 @@ _DEFAULT_BINDING = { # File Menu
                      ed_glob.ID_PRE_MARK : (u"Alt", u"Left"), # Win/Linux
                      ed_glob.ID_SHOW_SHELF : (u"Ctrl", u"Alt", u"S"),
                      ed_glob.ID_PANELIST : (u"Alt", u"1"), # Win/Linux
-                     ed_glob.ID_MAXIMIZE_EDITOR : (u"Ctrl", u"M"), 
+                     ed_glob.ID_MAXIMIZE_EDITOR : (u"Ctrl", u"M"),
 
                      # Format Menu
                      ed_glob.ID_TOGGLECOMMENT : (u"Ctrl", u"1"),
@@ -1039,7 +1071,6 @@ _DEFAULT_BINDING = { # File Menu
 
                      # Tools Menu
                      ed_glob.ID_COMMAND : (u"Ctrl", u"E"),
-                     ed_glob.ID_KWHELPER : (u"Ctrl", u"K"),
                      ed_glob.ID_RUN_LAUNCH : (u"F5",),
                      ed_glob.ID_LAUNCH_LAST : (u"Shift", u"F5")
                      }
